# Bins the data points of an AEG (az-el-gain) file into two target zones and one non-target zone.
#
# Outputs:
#    Difference.dat - statistics for annotation on the map with GMT's pstext.
# 
# Call:
#    gawk -f AEGbin.awk <target.xy> <filename.AEG>
#
# The target zones used to calculate statistics are defined by the file target.xy
# This file has two lines each containing 4 integers and a word:
#    Target 1: az-left to az-right, el-lo to el-hi, name (one word)
#    Target 2: az-left to az-right, el-lo to el-hi, name (one word)
# If a target zone is not needed, set the azimuths or elevations equal to each other.
# Both lines are REQUIRED, even if target zone(s) are not needed.
#
# NOTE: Points in Target zone 1 are EXCLUDED from the statistics for Target zone 2.
#
# 03 Sep 20: v1.0 - Created.
# 03 Oct 21: v1.1 - Changed name of maximum/minimum change in change to "largest increase" and "largest decrease".
# ===================================================================================================================
BEGIN {
   AEGBinVersion = "AEGBin v1.1 - 2003 Oct 21 Tue"
   print AEGBinVersion

   GAINFLOOR = -15.0

   # Logical constants
   TRUE = 1
   FALSE = 0

   # Table index names
   EXISTS = 1
   LEFT = 2
   RIGHT =3
   LOW = 4
   HIGH = 5
   NAME = 6
   INCLUDESNORTH = 7
   NRDATAPOINTS = 8
   MAXDB = 9
   MAXDBAZ = 10
   MAXDBEL = 11
   MINDB = 12
   MINDBAZ = 13
   MINDBEL = 14

   Azimuth = 0.0
   Elevation = 0.0
   Gain = 0.0
   high = 0.0
   left = 0.0
   low = 0.0
   NrDataPoints = 0.0
   right = 0.0
   StatisticsFile = "Difference.dat"

   # Initialize array for two target zones and a third, untargeted region.
   for (i=1; i<=3; i++)
   {
      if (i<3) Target[ i, EXISTS] = FALSE
      else Target[ i, EXISTS] = TRUE
      Target[ i, LEFT] = 0.0
      Target[ i, RIGHT] = 0.0
      Target[ i, LOW] = 0.0
      Target[ i, HIGH] = 0.0
      Target[ i, INCLUDESNORTH] = FALSE
      Target[ i, NRDATAPOINTS] = 0.0
      Target[ i, MAXDB] = -200		# Biggest positive change in dB is from -99.99 to +99.99
      Target[ i, MINDB] = +200
   }

   # Now get the zone boundaries from the first input file (target.xy).
   for (i=1; i<=2; i++)
   {
      getline
      left = $1
      right = $2
      low = $3
      high = $4	
      if ( (left != right) && (low != high) )
      {
         Target[ i, EXISTS] = TRUE
         Target[ i, LEFT] = left
         Target[ i, RIGHT] = right
         Target[ i, LOW] = low
         Target[ i, HIGH] = high
         Target[ i, NAME] = $5
         if ( left > right ) Target[ i, INCLUDESNORTH] = TRUE
      }
   }
   Target[ 3, NAME] = "Non-targeted area"
}

# ===================================================================================================================
# Line-by-line AEG output file processing
# This part of a GAWK command program is looped through until the entire contents of all remaining files have been processed.
{
   # Force conversion to numbers.
   Azimuth = $1
   Elevation = $2
   Gain = $3

   # Is this point within target zone 1, 2 or the non-target area (zone 3)?
   InTarget = 3   # Begin by assuming point is not in a target.

   # Check first for Zone 2, then zone 1, since a point in zone 1 is not considered part of zone 2.
   for( i=2; i>=1; i--)
   {
      # Check if within range of elevations
      if ( (Target[ i, EXISTS] == TRUE) &&
         ( Elevation >= Target[ i, LOW ] ) &&
         ( Elevation <= Target[ i, HIGH] ) )
      {
         # Check if within range of azimuths
         if ( Target[ i, INCLUDESNORTH] == TRUE )
         {
            if ( ( Azimuth >= Target[ i, LEFT ] ) || ( Azimuth <= Target[ i, RIGHT] ) ) InTarget = i
         }
         else if ( ( Azimuth >= Target[ i, LEFT ] ) && ( Azimuth <= Target[ i, RIGHT] ) ) InTarget = i
      }
   }

   Target[ InTarget, NRDATAPOINTS]++

   # See if the new datapoint gain is a maximum or minimum within the target zone.
   if ( Gain > Target[ InTarget, MAXDB] )
   {
      Target[ InTarget, MAXDB] = Gain
      Target[ InTarget, MAXDBAZ] = Azimuth
      Target[ InTarget, MAXDBEL] = Elevation
   }
   if ( Gain < Target[ InTarget, MINDB] )
   {
      Target[ InTarget, MINDB] = Gain
      Target[ InTarget, MINDBAZ] = Azimuth
      Target[ InTarget, MINDBEL] = Elevation
   }

   # To calculate median statistics, power or gain values in each zone must be in an ordered list.
   # Take advantage of NEC's granularity in calculating gain to a precision of 0.01 dBi.
   # Bin the gain values from the floor to the maximum gain in the target zone, counting the number of times each gain value appears.
   # This approach takes less space than an ordered list of 20,629 points.

   # GAWK arrays and array indices are strings.  Thus, the initial value of an array position is "", not numeric zero.
   # To avoid confusion, force the index to be an integer that will always get converted into the same kind of string.
   GainIndex = int( 100 * Gain)

   # Bump up the count in the appropriate gain bin.
   # Use one dimension arrays for each target zone so that the convention "if (value in array)" can be used later.
   if (InTarget == 1) TargetGain1[ GainIndex]++
   if (InTarget == 2) TargetGain2[ GainIndex]++
   if (InTarget == 3) TargetGain3[ GainIndex]++
}

# ===================================================================================================================
END {
   printf "\nTarget 1 contains %i data points.\n", Target[ 1, NRDATAPOINTS]
   printf "Target 2 contains %i data points.\n", Target[ 2, NRDATAPOINTS]
   printf "Non-target contains %i data points.\n", Target[ 3, NRDATAPOINTS]
   printf "Total data points binned: %i.\n\n", Target[ 1, NRDATAPOINTS] + Target[ 2, NRDATAPOINTS] + Target[ 3, NRDATAPOINTS]

   printf ">  +0.00 3.50 8 0 0 TL 9p 4.00 l\n" >StatisticsFile
   for ( i=1; i<=3; i++)
   {
      if (Target[ i, EXISTS] == TRUE)
      {
         # Median is the middle figure in the ordered list.
         MidPoint = int( Target[ i, NRDATAPOINTS] / 2)

         # Now go through the existing bins once, starting at the floor gain value, to locate the median value.
         PointsFound = 0.0
         for ( GainIndex = int( 100 * Target[ i, MINDB] ); PointsFound <= MidPoint; GainIndex++)
         {
            # How many points have this value?
            NrPoints = 0
            if ( i==1 && GainIndex in TargetGain1) NrPoints = TargetGain1[ GainIndex]
            if ( i==2 && GainIndex in TargetGain2) NrPoints = TargetGain2[ GainIndex]
            if ( i==3 && GainIndex in TargetGain3) NrPoints = TargetGain3[ GainIndex]
            PointsFound += NrPoints
         }		# Fall out of the loop when we have equaled or past the midpoint of the list.
         MedianGain = (GainIndex -1)/100.00

         printf "%s:\n", Target[ i, NAME]
         printf "   Largest increase %+5.2f dB at %6.2f az, %5.2f el.\n", Target[ i, MAXDB], Target[ i, MAXDBAZ], Target[ i, MAXDBEL]
         printf "   Largest decrease %+5.2f dB at %6.2f az, %5.2f el.\n", Target[ i, MINDB], Target[ i, MINDBAZ], Target[ i, MINDBEL]
         printf "   median %+5.2f\n", MedianGain

         # Write the statistics for annotation on maps.
         if ( i <  3) printf "Target %1i - %s: %3i-%3i\217 az  %2i-%2i\217 el.\n\n", i, Target[ i,NAME], Target[ i,LEFT], Target[ i,RIGHT], Target[ i,LOW], Target[ i,HIGH] >StatisticsFile
         if ( i == 3) printf "Non-targeted area statistics:\n\n" >StatisticsFile
         printf "\\040\\040\\040Largest increase: %+5.2f dB at %3i\217 az  %2i\217 el.\n\n", Target[ i, MINDB], Target[ i, MINDBAZ], Target[ i, MINDBEL] >StatisticsFile
         printf "\\040\\040\\040Largest decrease: %+5.2f dB at %3i\217 az  %2i\217 el.\n\n", Target[ i, MAXDB], Target[ i, MAXDBAZ], Target[ i, MAXDBEL] >StatisticsFile
         printf "\\040\\040\\040Median difference: %+5.2f dB.\n\n\n", MedianGain >StatisticsFile
      }
   }
   # Footnote the minimum gain used in statistics within zones.
   printf "* Gain floor for calculating differences is %+3.1f dBi.\n\n", GAINFLOOR >StatisticsFile
   printf "NEC x-axis is North azimuth.\n\n" >StatisticsFile
   printf "Statistics computed by %s.\n", AEGBinVersion >StatisticsFile

   close( StatisticsFile)
}
