# Substracts the gain between two antenna patterns.  Generates the following ASCII text files:
#    Difference.AEG -- azimuth, elevation, and delta-gain table.
#    FloorMask.xy -- rectangles of consecutive azimuths at each elevation within which at least one pattern's gain was => floor gain.
# 
# Call:
#    gawk -f NOUDifference.awk <filename1.aeg> <filename2.aeg>
#
# NOTE: The variable LowestGainForAveraging is set to the lowest gain that will be used in averaging.
# This recognizes that real-world construction can not maintain tolerances and stability necessary for very low gain levels,
# e.g., more than 30 dB below the main beam of a large yagi.  Environmental scattering will also fill in deep nulls in patterns.
#
# NOTE!! Azimuths in the AEG files begin at 360 and run down to 0.
#
# 03 Sep 16: v1.0 Created.
# ===================================================================================================================
BEGIN {
   NOUDifferenceVersion = "NOUDifference.awk v1.0 - 2003 Sep 16 Tue"
   print NOUDifferenceVersion

   Azimuth = 0.0
   DataPointNr = 1
   Elevation = 0.0
   FloorMaskFilename = "FloorMask.xy"
   Gain = 0.0
   LowestGain = -15.0		# Floor value for practical gain.
   OutputFilename = "Difference.AEG"
   PreviousAzimuth = 360.0
   PreviousElevation = 0.0		# Tracks elevation; when jumps down to 0 again, we must be in second pattern file.
   State = "First Pattern"	# Indicates which pattern we think we are processing.

   TRUE = 1.0
   FALSE = 0.0
}

# ===================================================================================================================
# Line-by-line AEG file processing
# This part of a GAWK command program is looped through until the entire contents of all remaining files have been processed.
{
   Azimuth = $1		# force format conversion to numbers.
   Elevation = $2
   Gain = $3

   if (Elevation < PreviousElevation)  # If elevation goes backwards, we have started reading the second file.
   {
      printf "%i data points found in first pattern.\n", DataPointNr - 1
      DataPointNr = 1
      State = "Second Pattern"
   }

   if ( State == "First Pattern")
   {
      # Load up the arrays with pattern #1's data values.
      Pattern1Azimuth[ DataPointNr] = Azimuth
      Pattern1Elevation[ DataPointNr] = Elevation
      if ( Gain > LowestGain) Pattern1Gain[ DataPointNr] = Gain
      else Pattern1Gain[ DataPointNr] = LowestGain		# Use the floor value if pattern gain is below the floor.
      DataPointNr++
      PreviousElevation = Elevation
   }

   if ( State == "Second Pattern")
   {
      if( ( Pattern1Azimuth[ DataPointNr] == Azimuth) && ( Pattern1Elevation[ DataPointNr] == Elevation) )		# Verify the data points match in azimuth & elevation.
      {
         if ( Gain < LowestGain) Gain = LowestGain		# Use the floor value if pattern gain is below the floor.

         # If this elevation below previous, we just started second data set.
         if ( Elevation < PreviousElevation)
         {
            MaskLowerEl = 0.0				# The first row's mask boxes have a lower elevation of 0.
            MaskUpperEl = 2 * Elevation			# ... and an upper elevation that brackets the data points.
            FirstAzInRow = TRUE				# Set flag indicating we've started a new row of data points.
            InMask = FALSE					# Assume temporarily that this data point is outside the mask.
         }

         # if this elevation above previous, we Just jumped up to the next row within the second data set.
         if ( Elevation > PreviousElevation)
         {
            # Were we inside a set of valid data at the end of the previous row?
            if (InMask == TRUE)
            {
               # Write out a description of the last mask of the row.
               printf "%6.2f %5.2f\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename	# Spell out the four corners of a box ending at 0 azimuth.
               printf "%6.2f %5.2f\n", MaskLeftAz, MaskUpperEl >FloorMaskFilename 
               printf "  0.00 %5.2f\n", MaskUpperEl >FloorMaskFilename
               printf "  0.00 %5.2f\n", MaskLowerEl >FloorMaskFilename
               printf "%6.2f %5.2f\n>\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename 	# Close the box and write '>' on a separate line to tell GMT the next polygon follows.
            }
            MaskLowerEl = MaskUpperEl			# Set this row's mask boxes to start where the previous row left off.
            MaskUpperEl = MaskLowerEl + ( (Elevation - MaskLowerEl) * 2)      # And set the top of the row's mask boxes to bracket the data point elevations.
            if (MaskUpperEl > 89) MaskUpperEl = 90.0	# Force to include the zenith in the last row of data.
            FirstAzInRow = TRUE				# Set flag indicating we've started a new row of data points.
            InMask = FALSE					# Assume temporarily that this data point is outside the mask.
         }

         # if at the same elevation, check to see if we just passed 180 azimuth.
         # gmt has trouble dealing with polygons which cross north and 180.
         if ( ( Elevation == PreviousElevation) && ( PreviousAzimuth >= 180) && (Azimuth <= 180) && (InMask == TRUE) )
         {
            # Spell out the four corners of a box ending at 180 azimuth.
            # Azimuths must be in the range -180 to +180.
            printf "%6.2f %5.2f\n", MaskLeftAz-360, MaskUpperEl >FloorMaskFilename 
            printf "%6.2f %5.2f\n", MaskLeftAz-360, MaskLowerEl >FloorMaskFilename
            printf "-180.00 %5.2f\n", MaskLowerEl >FloorMaskFilename
            printf "-180.00 %5.2f\n", MaskUpperEl >FloorMaskFilename
            printf "%6.2f %5.2f\n", MaskLeftAz-360, MaskUpperEl >FloorMaskFilename 
            printf ">\n" >FloorMaskFilename 	# Close the box and write '>' on a separate line to tell GMT the next polygon follows.

            # Start a new mask at 180 azimuth.
            MaskLeftAz = 180.0
         }

         if ( ( Gain == LowestGain) && ( Pattern1Gain[ DataPointNr] == LowestGain) )
         {
            # Both patterns have gains at or below the floor level.

            # Set the difference to zero.
            printf "%6.2f %5.2f 0\n", Azimuth, Elevation >OutputFilename

            # Check to see if the previous data point in this row was above the gain floors and inside a mask.
            if (InMask == TRUE)
            {
               # No longer inside a mask.
               InMask = FALSE
               MaskRightAz = (PreviousAzimuth + Azimuth) / 2	# Set the right edge of the mask box halfway between the last and current azimuth.
               # Write out a description of the last mask.
               # Azimuths must be in the range -180 to +180 for masks.
               if (MaskLeftAz > 180) MaskLeftAz -= 360
               if (MaskRightAz > 180) MaskRightAz -= 360
               printf "%6.2f %5.2f\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename	# Spell out the four corners of the box.
               printf "%6.2f %5.2f\n", MaskLeftAz, MaskUpperEl >FloorMaskFilename
               printf "%6.2f %5.2f\n", MaskRightAz, MaskUpperEl >FloorMaskFilename
               printf "%6.2f %5.2f\n", MaskRightAz, MaskLowerEl >FloorMaskFilename
               printf "%6.2f %5.2f\n>\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename 	# Close the box and write '>' on a separate line to tell GMT the next polygon follows.
            }
         }
         else
         {
            # At least one pattern was above the gain floor.

            # Write out the difference.
            printf "%6.2f %5.2f %6.2f\n", Azimuth, Elevation, Gain - Pattern1Gain[ DataPointNr] >OutputFilename

            # Check to see if the previous data point in this row was not inside a mask.
            if (InMask == FALSE)
            {
               # Starting a new mask.
               InMask = TRUE
               if (FirstAzInRow == TRUE)   MaskLeftAz = 360.0	# If we just started a new row, then the mask starts at North.
               else MaskLeftAz = (PreviousAzimuth + Azimuth) / 2	# Otherwise set the left edge of the mask box halfway between the last and current azimuth.
            }
         }
      }

      else		# Error: data points did not match in azimuth or elevation.  Inform user.  Ignore data point.
      {
         printf "Az1 %6.2f El1 %5.2f != Az2 %6.2f El2 %5.2f.  Can't subtract pattern gains.\n", Pattern1Azimuth[ DataPointNr], Azimuth, Pattern1Elevation[ DataPointNr], Elevation
         exit 1
      }

      DataPointNr++
      PreviousElevation = Elevation
      PreviousAzimuth = Azimuth
      FirstAzInRow = FALSE
   }	# End of processing a data point from the second file.
}

# ===================================================================================================================
END {
   if (InMask == TRUE)  # Need to wrap up the last mask box.
   {
      # Write out a description of the last mask of the row.
      printf "%6.2f %5.2f\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename	# Spell out the four corners of a box ending at 0 azimuth.
      printf "%6.2f %5.2f\n", MaskLeftAz, MaskUpperEl >FloorMaskFilename 
      printf "  0.00 %5.2f\n", MaskUpperEl >FloorMaskFilename
      printf "  0.00 %5.2f\n", MaskLowerEl >FloorMaskFilename
      printf "%6.2f %5.2f\n>\n", MaskLeftAz, MaskLowerEl >FloorMaskFilename 	# Close the box and write '>' on a separate line to tell GMT the next polygon follows.
   }

   printf "%i data points found in second pattern.\n", DataPointNr - 1
   close( OutputFilename)
}
