   echo off

REM Produces full sky-hemisphere plot of the difference between two antenna patterns.
REM Antenna pattern #2 is subtracted from pattern #1.
REM Annotates plot with comments from the NEC input card decks from each model run.
REM Annotates plot with key statistics, including variations found within 3 zones defined by the target.xy file.
REM Any points with pattern gain of <-15 dBi are assumed to be -15 dBi.
REM Color scale DeltadB.cpt (org-red-white-grn-turquoise) used to color map.
REM
REM Call:
REM   NOUDifference <filename #1> <filename #2>
REM   Filenames are the .AEG tables generated by NOUPlot.bat
REM
REM NOUDifference assumes that NOUPlot has been run previously on each NEC output, and has generated the following files:
REM   *.cm -- comment cards extracted from NEC input card deck.
REM   *.AEG -- pattern tables of Az, El, and gain in binary form (double precision).
REM 
REM The target zones used to calculate statistics are defined by the file target.xy
REM This file has two lines each containing 4 integers and a word:
REM    Target 1: az-left to az-right, el-lo to el-hi, name (one word)
REM    Target 2: az-left to az-right, el-lo to el-hi, name (one word)
REM If a target zone is not needed, set the azimuths or elevations equal to each other.
REM Both lines are REQUIRED, even if target zone(s) are not needed.
REM 
REM NOTE: Points in Target zone 1 are EXCLUDED from the statistics for Target zone 2.
REM
REM Utilities used by this program:
REM    NOUDifference.awk -- does the pattern subtraction and other homework.
REM    target.awk -- reads the target.xy file and generates inputs for target area masking and descriptions.
REM
REM Files used in generating the plot:
REM    AxisLabel.txt -- annotations around the edge of the map.
REM    ContinentBoxesW3.xy -- outlnes of continental boxes from W3.
REM    ContinentsW3.xy -- continent names from W3.
REM    DeltaDb.cpt -- color scheme for coloring the map.
REM    DeltaDb_scale.cpt -- color scheme for generating the legend.
REM    target.xy -- describes up to 2 target zones.
REM    targethorizontal.xy -- horizontal boundary descriptions for drawing the zones.
REM    targetvertical.xy -- vertical boundary descriptions for drawing the zones.
REM 
REM Output:
REM    Difference.ps -- annotated plot of difference between two antenna patterns.
REM 
REM 03 Sep 15: v1.0 created.
REM 03 Oct 21: v1.1 Use gray pen rather than white to plot target zone rectangles.

REM Version number control.
    echo NOUDifference.bat v1.1 - 2003 Oct 21 Tue

REM Remove any old Difference.* files.
    del Difference.*

REM Subtract pattern #2 from pattern #1 and save it in Difference.AEG.
REM Also create FloorMask.xy with coordinates of boxes where either pattern's gain is > floor gain of -15 dBi.
    gawk -f NOUDifference.awk %1.AEG %2.AEG

REM Convert the .AEG and .xy files into binary format for faster processing.
    gmtconvert Difference.AEG -bo >Difference.bin
    gmtconvert FloorMask.xy -bo2 -M >FloorMask.bin

REM Massage the data into an even grid.
    blockmedian Difference.bin -bi -I1 -L -R-180/180/0/90 -bo >Difference.b
    surface Difference.b -bd -GDifference.grd -I1 -R-180/180/0/90 -C0.05

REM Because surface creates a global interpolation, values will wiggle around 0 in areas where the patterns were both <= floor gain.
REM Create a floor mask with zero at grid points where the patterns were both <= floor gain.
    grdmask FloorMask.bin -GFloorMask.grd -bi2 -I1 -M -R-180/180/0/90 -N0/1/1

REM Zero out the areas below the floor.
    grdmath Difference.grd FloorMask.grd MUL = Difference.grd

REM The data plot will be in the top half of the screen; therefore use -Xa0.25i -Ya4.50i.
REM plot the data in top half of screen, using DeltadB.cpt to color.
    gmtset UNIX_TIME_POS 0i/3.375i
    gmtset BASEMAP_FRAME_RGB 160/160/160
    gmtset GRID_CROSS_SIZE 0.05i
    grdview Difference.grd -JKf0/10.5i -Ba15g15/a10g10S -CDeltadB.cpt -K -Lg -Qi -R-180/180/0/90 -U"\216 Eric Scace K3NA.  Filenames: %1 - %2" -Xa0.25i -Ya4.50i >>Difference.ps

REM Plot the contours.  Don't plot the 0 dB contour; instead plot -1 and +1 dB.
    gmtset BASEMAP_FRAME_RGB 0/0/0
    grdcontour Difference.grd -A3f6t -JKf0/10.5i -R-180/180/0/90 -G2i -K -L3/99 -O -W0.25p/0/0/0 -Xa0.25i -Ya4.50i >>Difference.ps
    grdcontour Difference.grd -A1f6t -JKf0/10.5i -R-180/180/0/90 -G2i -K -L1/1.5 -O -W0.25p/0/0/0 -Xa0.25i -Ya4.50i >>Difference.ps
    grdcontour Difference.grd -A3f6t -JKf0/10.5i -R-180/180/0/90 -G2i -K -L-99/-3 -O -W0.25p/0/0/0/ta -Xa0.25i -Ya4.50i >>Difference.ps
    grdcontour Difference.grd -A1f6t -JKf0/10.5i -R-180/180/0/90 -G2i -K -L-1/-0.5 -O -W0.25p/0/0/0/ta -Xa0.25i -Ya4.50i >>Difference.ps

REM Plot the color scale. Note: DeltadB_scale.cpt file is used so that the scale ends at -21 dBi and markers appear at correct places.
    psscale -D8.00i/3.50i/5.0i/0.125ih -Bg1a3:"change in gain [dB]": -CDeltadB_scale.cpt -K -O -Xa0.25i -Ya4.50i >>Difference.ps

REM Anotate axes on sides, top.  Note: GMT doesn't anotate these values, or incorrectly positions the labels, so we must add this manually.
REM Use -X, -Y offsets to be 1/4 inch to the left, and 3/4 inch below of the map's plotted location.
REM -R definition places 0,0 at lower left corner of the map itself.  Text plotting locations are therefore relative to the map's lower left corner.
    pstext AxisLabel.txt -JX11i/3.75i -R-0.25/10.75/-0.75/3 -G160 -K -O -Xa0i -Ya3.75i >>Difference.ps

REM Plot continental azimuth ranges, using a series of boxes and then adding the titles.
    psxy ContinentBoxesW3.xy -JX11i/3.75i -R-0.25/10.75/-0.75/3 -K -L -M -O -W0.25/160 -Xa0i -Ya3.75i >>Difference.ps
    pstext ContinentsW3.txt -JX11i/3.75i -R-0.25/10.75/-0.75/3 -C0.02 -G160 -K -O -Xa0i -Ya3.75i >>Difference.ps

REM Prepare a description of the target areas in Target1.xy and Target2.xy files
    gawk -f target.awk target.xy

REM Draw target areas with white pen
    psxy targetvertical.xy -JKf0/10.5i -R-180/180/0/90 -K -M -O -W0.25p/127/127/127 -Xa0.25i -Ya4.50i >>Difference.ps
    psxy targethorizontal.xy -JKf0/10.5i -R-180/180/0/90 -A -K -M -O -W0.25p/127/127/127 -Xa0.25i -Ya4.50i >>Difference.ps

REM Note: each annotation area is controlled by the pstext boundaries AND by commands contained in the first line of the .CM, .CON and .DAT files.
REM To change column widths, one must change both the pstext and the first line of those files.
REM Annotate with NEC comment cards for pattern #1 below figure in lower left column.
REM hint: Add -B to view border of print area.
    pstext %1.CM -JX3.5i/3.5i -R0/3.5/0/3.5 -K -M -O -Xa0.25i -Ya0.25i >>Difference.ps

REM Annotate with NEC comment cards for pattern #2 in center column.
    pstext %2.CM -JX3.5i/3.5i -R0/3.5/0/3.5 -K -M -O -Xa3.75i -Ya0.25i >>Difference.ps

REM Sort the data points into target and non-target bins.
REM Find max and min locations and values in each bin.
    gawk -f AEGBin.awk target.xy Difference.AEG

REM Display statistics below figure in lower right column (use placeholder file).
REM hint: Add -B to view border of print area.
REM No -K: this is the last item plotted.
    pstext Difference.dat -JX3.5i/3.5i -R0/4/0/3.5 -M -O -Xa7.25i -Ya0.25i >>Difference.ps

REM Finished.  Remove temporary files.
    del FloorMask.*
    del Difference.AEG
    del Difference.bin
    del Difference.b
    del Difference.dat
    del Difference.grd
    del targetvertical.xy
    del targethorizontal.xy
