# Reads specified .xy file, describing target zones 1 and 2.
# Creates vertical and horizontal plot line specifications for the target zones for plotting on the sky hemisphere:
#    TargetHorizontal.xy
#    TargetVertical.xy
# Creates .xy files for the two rectangles that can be used in GMT grdmask and grdmath processing:
#    Target1.xy
#    Target2.xy
# 
# Call:
#    gawk -f target.awk <target.xy>
#
# The target zones used to calculate statistics are defined by the file target.xy
# This file has two lines each containing 4 integers and a word:
#    Target 1: az-left to az-right, el-lo to el-hi, name (one word)
#    Target 2: az-left to az-right, el-lo to el-hi, name (one word)
# If a target zone is not needed, set the azimuths or elevations equal to each other.
# Both lines are REQUIRED, even if target zone(s) are not needed.
#
# NOTE: Points in Target zone 1 are EXCLUDED from the statistics for Target zone 2.
#
# 03 Sep 16: v1.0 - Created.
# ===================================================================================================================
BEGIN {
   print "Target.awk v1.0 2003 Sep 16 Tue\n"
   i=1
}

# This part of a GAWK command program is looped through until the entire contents of all remaining files have been processed.
{
   left = $1
   right = $2
   low = $3
   high = $4	

   if (i==1) OutFile = "Target1.xy"
   else OutFile = "Target2.xy"

   if ( (left != right) && (low != high) )	# Does a target exist?
   {
      # Write an accumulating description of all target borders for GMT to plot.
      # Vertical lines are great circle lines that follow longitude (azimuth).
      printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, low, left, high >"targetvertical.xy"    # great circle route up left longitude.
      printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", right, low, right, high >"targetvertical.xy"    # great circle route up right longitude.
      # Horizontal lines are non-great circles lines that follow latitude (elevation).  A separate GMT psxy statement is needed for these.
      printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, high, right, high >"targethorizontal.xy"    # straight across the top.
      printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, low, right, low >"targethorizontal.xy"    # straight across the bottom.
   }

   # Write a .xy file containing the coordinates.
   if ( (left <= 360) && (left >= 180) && (right >= 0) && (right <= 180) )		# Does target straddle north?
   {		# Yes.  Modify longitude to wrap.
      printf "%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n", left, low, left, high, right+360, high, right+360, low, left, low >OutFile
   }
   else
   {
      if ( (left < 180) && (right > 180) )	# Does target straddle south?
      {
         printf "%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n", left, low, left, high, 180, high, 180, low, left, low >OutFile
         printf ">%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n", right, low, right, high, 180.1, high, 180.1, low, right, low >OutFile
      }
      else
      {
         # rectangle does not straddle north or south.
         printf "%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n", left, low, left, high, right, high, right, low, left, low >OutFile
      }
   }

   # Next target zone.
   i++
}

# Check to see if both targets were defined.  If not, create a null target definition for target2.xy.
END {
   if (i <= 2) printf "%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n%5.1f %4.1f\n", 0, 90, 0, 90, 0, 90, 0, 90, 0, 90 >"Target2.xy"
}